/*
 * QLogic Fibre Channel HBA Driver
 * Copyright (c)  2003-2013 QLogic Corporation
 *
 * See LICENSE.qla2xxx for copyright and licensing details.
 *
 * PCI searching functions pci_get_domain_bus_and_slot & pci_channel_offline
 * Copyright (C) 1993 -- 1997 Drew Eckhardt, Frederic Potter,
 *                                      David Mosberger-Tang
 * Copyright (C) 1997 -- 2000 Martin Mares <mj@ucw.cz>
 * Copyright (C) 2003 -- 2004 Greg Kroah-Hartman <greg@kroah.com>.
 */

#ifndef __QLA_KCOMPAT_H
#define __QLA_KCOMPAT_H
#include <linux/version.h>
#include <linux/pci.h>

#ifndef DID_TRANSPORT_DISRUPTED
#define DID_TRANSPORT_DISRUPTED DID_BUS_BUSY
#endif

#ifndef list_first_entry
#define list_first_entry(ptr, type, member) \
	list_entry((ptr)->next, type, member)
#endif


#if (RHEL_MINOR < 3) /* For RHEL 5.2 compatibility */
static inline int pci_channel_offline(struct pci_dev *pdev)
{
	return pdev->error_state != pci_channel_io_normal;
}
#endif

#if (RHEL_MINOR < 5) /* For RHEL 5.4 and below compatibility */
static inline struct pci_dev *pci_get_domain_bus_and_slot(int domain, unsigned int bus,
                                            unsigned int devfn)
{
        struct pci_dev *dev = NULL;

        while ((dev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, dev)) != NULL) {
                if (pci_domain_nr(dev->bus) == domain &&
                    (dev->bus->number == bus && dev->devfn == devfn))
                        return dev;
        }
        return NULL;
}
#endif

#endif

